import React, { useRef, useState } from "react";
import { Navbar, Nav, NavDropdown, Modal, Button } from "react-bootstrap";
import { useAuth } from "../context/auth";
import { useBank } from "../context/bank";


export default function Header() {
    const auth = useAuth();
    const bank = useBank();
    const [modalOpen, setModalOpen] = useState<boolean>(false);
    const bankAccountRef = useRef<HTMLInputElement>(null);
    const nameRef = useRef<HTMLInputElement>(null);

    const openBankSettings = () => {
        setModalOpen(true);
    }

    const closeModal = () => {
        setModalOpen(false);
    }

    const isValidInteger = (str) => {
        str = str.trim();
        if (!str) {
            return false;
        }
        str = str.replace(/^0+/, "") || "0";
        const n = Math.floor(Number(str));
        return n !== Infinity && String(n) === str && n >= 0;
    }

    const save = () => {
        if (bankAccountRef.current?.value) {
            try {
                // Validace kódu banky
                if (bankAccountRef.current?.value.indexOf('/') < 0) {
                    throw Error("Číslo účtu neobsahuje lomítko/kód banky")
                }
                const split = bankAccountRef.current?.value.split("/");
                if (split[1].length !== 4) {
                    throw Error("Kód banky musí být 4 číslice")
                }
                if (!isValidInteger(split[1])) {
                    throw Error("Kód banky není číslo")
                }

                // Validace čísla a předčíslí
                let cislo = split[0];

                if (cislo.indexOf('-') > 0) {
                    cislo = cislo.replace('-', '');
                }
                if (!isValidInteger(cislo)) {
                    throw Error("Předčíslí nebo číslo účtu neobsahuje pouze číslice")
                }
                if (cislo.length < 16) {
                    cislo = cislo.padStart(16, '0');
                }
                let sum = 0;
                for (var i = 0; i < cislo.length; i++) {
                    const char = cislo.charAt(i);
                    const order = (cislo.length - 1) - i;
                    const weight = (2 ** order) % 11;
                    sum += Number.parseInt(char) * weight
                }
                if (sum % 11 !== 0) {
                    throw Error("Číslo účtu je neplatné")
                }
            } catch (e) {
                alert(e.message)
                return
            }
        }
        bank?.setBankAccountNumber(bankAccountRef.current?.value);
        bank?.setBankAccountHolderName(nameRef.current?.value);
        closeModal();
    }

    return <Navbar variant='dark' expand="lg">
        <Navbar.Brand>Luncher</Navbar.Brand>
        <Navbar.Toggle aria-controls="basic-navbar-nav" />
        <Navbar.Collapse id="basic-navbar-nav">
            <Nav className="nav">
                <NavDropdown align="end" title={auth?.login} id="basic-nav-dropdown">
                    <NavDropdown.Item onClick={openBankSettings}>Nastavit číslo účtu</NavDropdown.Item>
                    <NavDropdown.Item onClick={auth?.clearLogin}>Odhlásit se</NavDropdown.Item>
                </NavDropdown>
            </Nav>
        </Navbar.Collapse>
        <Modal show={modalOpen} onHide={closeModal} size="lg">
            <Modal.Header closeButton>
                <Modal.Title>Bankovní účet</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <p>Nastavením čísla účtu umožníte automatické generování QR kódů pro úhradu za vámi provedené objednávky v rámci Pizza day.<br />Pokud vaše číslo účtu neobsahuje předčíslí, je možné ho zcela vynechat.<br /><br />Číslo účtu není ukládáno na serveru, posílá se na něj pouze za účelem vygenerování QR kódů.</p>
                Číslo účtu: <input className="mb-3" ref={bankAccountRef} type="text" placeholder="123456-1234567890/1234" defaultValue={bank?.bankAccount} /> <br />
                Název příjemce (jméno majitele účtu): <input ref={nameRef} type="text" placeholder="Jan Novák" defaultValue={bank?.holderName} />
            </Modal.Body>
            <Modal.Footer>
                <Button variant="secondary" onClick={closeModal}>
                    Storno
                </Button>
                <Button variant="primary" onClick={save}>
                    Uložit
                </Button>
            </Modal.Footer>
        </Modal>
    </Navbar>
}