import express from "express";
import { Server } from "socket.io";
import bodyParser from "body-parser";
import { fetchPizzy } from "./chefie";
import cors from 'cors';
import { addPizzaOrder, createPizzaDay, deletePizzaDay, finishPizzaDelivery, finishPizzaOrder, getData, lockPizzaDay, removePizzaOrder, unlockPizzaDay, updateChoice, updateNote } from "./service";
import dotenv from 'dotenv';
import path from 'path';
import { fetchMenus } from "./restaurants";
import { getQr } from "./qr";

const ENVIRONMENT = process.env.NODE_ENV || 'production'
dotenv.config({ path: path.resolve(__dirname, `../.env.${ENVIRONMENT}`) });

const app = express();
const server = require("http").createServer(app);
const io = new Server(server, {
    cors: {
        origin: "*",
    },
});

// Body-parser middleware for parsing JSON
app.use(bodyParser.json());
// app.use(express.json());

app.use(cors({
    origin: '*'
}));

/** Vrátí data pro aktuální den. */
app.get("/api/data", (req, res) => {
    res.status(200).json(getData());
});

/** Vrátí obědové menu pro dostupné podniky. */
app.get("/api/food", (req, res) => {
    fetchMenus().then(food => {
        res.status(200).json(food);
    })
});

/** Vrátí seznam dostupných pizz. */
app.get("/api/pizza", (req, res) => {
    fetchPizzy().then(pizzaList => {
        // console.log("Výsledek", pizzaList);
        res.status(200).json(pizzaList);
    });
});

/** Založí pizza day pro aktuální den, za předpokladu že dosud neexistuje. */
app.post("/api/createPizzaDay", (req, res) => {
    if (!req.body?.creator) {
        throw Error("Nebyl předán název zakládajícího");
    }
    const data = createPizzaDay(req.body.creator);
    res.status(200).json(data);
    io.emit("message", data);
});

/** Smaže pizza day pro aktuální den, za předpokladu že existuje. */
app.post("/api/deletePizzaDay", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login uživatele");
    }
    const data = deletePizzaDay(req.body.login);
    io.emit("message", data);
});

app.post("/api/addPizza", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login");
    }
    if (isNaN(req.body?.pizzaIndex)) {
        throw Error("Nebyl předán index pizzy");
    }
    const pizzaIndex = req.body.pizzaIndex;
    if (isNaN(req.body?.pizzaSizeIndex)) {
        throw Error("Nebyl předán index velikosti pizzy");
    }
    const pizzaSizeIndex = req.body.pizzaSizeIndex;
    fetchPizzy().then(pizzy => {
        if (!pizzy[pizzaIndex]) {
            throw Error("Neplatný index pizzy: " + pizzaIndex);
        }
        if (!pizzy[pizzaIndex].sizes[pizzaSizeIndex]) {
            throw Error("Neplatný index velikosti pizzy: " + pizzaSizeIndex);
        }
        const data = addPizzaOrder(req.body.login, pizzy[pizzaIndex], pizzy[pizzaIndex].sizes[pizzaSizeIndex]);
        io.emit("message", data);
        res.status(200).json({});
    })
});

app.post("/api/removePizza", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login");
    }
    if (!req.body?.pizzaOrder) {
        throw Error("Nebyla předána objednávka");
    }
    const data = removePizzaOrder(req.body.login, req.body?.pizzaOrder);
    io.emit("message", data);
    res.status(200).json({});
});

app.post("/api/lockPizzaDay", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login");
    }
    const data = lockPizzaDay(req.body.login);
    io.emit("message", data);
    res.status(200).json({});
});

app.post("/api/unlockPizzaDay", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login");
    }
    const data = unlockPizzaDay(req.body.login);
    io.emit("message", data);
    res.status(200).json({});
});

app.post("/api/finishOrder", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login");
    }
    const data = finishPizzaOrder(req.body.login);
    io.emit("message", data);
    res.status(200).json({});
});

app.post("/api/finishDelivery", (req, res) => {
    if (!req.body?.login) {
        throw Error("Nebyl předán login");
    }
    const data = finishPizzaDelivery(req.body.login, req.body.bankAccount, req.body.bankAccountHolder);
    io.emit("message", data);
    res.status(200).json({});
});

app.post("/api/updateChoice", (req, res) => {
    if (!req.body.hasOwnProperty('name')) {
        res.status(400).json({});
    }
    const data = updateChoice(req.body.name, req.body.choice);
    io.emit("message", data);
    res.status(200).json(data);
});

app.get("/api/qr", (req, res) => {
    if (!req.query?.login || typeof req.query.login !== 'string') {
        throw Error("Nebyl předán login");
    }
    const img = getQr(req.query.login);
    res.writeHead(200, {
        'Content-Type': 'image/png',
        'Content-Length': img.length
    });
    res.end(img);
});

app.post("/api/updateNote", (req, res) => {
    if (!req.body.login) {
        throw Error("Nebyl předán login");
    }
    if (req.body.note && req.body.note.length > 100) {
        throw Error("Poznámka může mít maximálně 100 znaků");
    }
    const data = updateNote(req.body.login, req.body.note);
    io.emit("message", data);
    res.status(200).json(data);
});

io.on("connection", (socket) => {
    console.log(`New client connected: ${socket.id}`);

    socket.on("message", (message) => {
        io.emit("message", message);
    });

    socket.on("disconnect", () => {
        console.log(`Client disconnected: ${socket.id}`);
    });
});

const PORT = process.env.PORT || 3001;
const HOST = process.env.HOST || '0.0.0.0';

server.listen(PORT, () => {
    console.log(`Server listening on ${HOST}, port ${PORT}`);
});

console.log(process.env.API_KEY)